#include <GL/glut.h>
#include <math.h>
#include "textures.h"
#include "labyrinth.h"
#include "draw.h"
#include "labyrinth_math.h"

extern GLuint texture[NUM_TEXTURES];
extern state s;

/**
 * Function that draws a part of the floor
 */
void drawFloor(int x, int z)
{
	glBindTexture(GL_TEXTURE_2D,texture[4]);
	!s.wireframe ? glBegin(GL_QUADS) : glBegin(GL_LINE_LOOP);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2d(0.0 + (x / (float)BOARD_WIDTH), 1.0 - (z / (float)BOARD_WIDTH)); glVertex3f(0.0, 0.0, 0.0);
		glTexCoord2d(0.0 + ((x + 1) / (float)BOARD_WIDTH), 1.0 - (z / (float)BOARD_WIDTH)); glVertex3f(SQUARE_SIZE, 0.0, 0.0);
		glTexCoord2d(0.0 + ((x + 1) / (float)BOARD_WIDTH), 1.0 - ((z + 1) / (float)BOARD_WIDTH)); glVertex3f(SQUARE_SIZE, 0.0, SQUARE_SIZE);
		glTexCoord2d(0.0 + (x / (float)BOARD_WIDTH), 1.0 - ((z + 1) / (float)BOARD_WIDTH)); glVertex3f(0.0, 0.0, SQUARE_SIZE);
	glEnd();
}

/**
 * Function that draws a solid wall
 */
void drawWall()
{
	float halfsize = SQUARE_SIZE / 2;
	glPushMatrix();
		/* Do a translation for easier drawing */
		glTranslatef(0.0, -halfsize, SQUARE_SIZE);
		!s.wireframe ? glBegin(GL_QUADS) : glBegin(GL_LINE_LOOP);
			/* front */
			glNormal3f(0.0, 0.0, 1.0);
			glVertex3f(0.0, 0.0, 0.0);
			glVertex3f(SQUARE_SIZE, 0.0, 0.0);
			glVertex3f(SQUARE_SIZE, -halfsize, 0.0);
			glVertex3f(0.0, -halfsize, 0.0);
		glEnd();
		!s.wireframe ? glBegin(GL_QUADS) : glBegin(GL_LINE_LOOP);
			/* back */
			glNormal3f(0.0, 0.0, -1.0);
			glVertex3f(SQUARE_SIZE, 0.0, -SQUARE_SIZE);
			glVertex3f(0.0, 0.0, -SQUARE_SIZE);
			glVertex3f(0.0, -halfsize, -SQUARE_SIZE);
			glVertex3f(SQUARE_SIZE, -halfsize, -SQUARE_SIZE);
		glEnd();
		!s.wireframe ? glBegin(GL_QUADS) : glBegin(GL_LINE_LOOP);
			/* top */
			glNormal3f(0.0, 1.0, 0.0);
			glVertex3f(0.0, 0.0, -SQUARE_SIZE);
			glVertex3f(SQUARE_SIZE, 0.0, -SQUARE_SIZE);
			glVertex3f(SQUARE_SIZE, 0.0, 0.0);
			glVertex3f(0.0, 0.0, 0.0);
		glEnd();
		!s.wireframe ? glBegin(GL_QUADS) : glBegin(GL_LINE_LOOP);
			/* left */
			glNormal3f(-1.0, 0.0, 0.0);
			glVertex3f(0.0, 0.0, -SQUARE_SIZE);
			glVertex3f(0.0, 0.0, 0.0);
			glVertex3f(0.0, -halfsize, 0.0);
			glVertex3f(0.0, -halfsize, -SQUARE_SIZE);
		glEnd();
		!s.wireframe ? glBegin(GL_QUADS) : glBegin(GL_LINE_LOOP);
			/* right */
			glNormal3f(1.0, 0.0, 0.0);
			glVertex3f(SQUARE_SIZE, 0.0, 0.0);
			glVertex3f(SQUARE_SIZE, 0.0, -SQUARE_SIZE);
			glVertex3f(SQUARE_SIZE, -halfsize, -SQUARE_SIZE);
			glVertex3f(SQUARE_SIZE, -halfsize, 0.0);
		glEnd();
	glPopMatrix();
}

/**
 * Function that draws a hole
 */
void drawHole()
{
	coord2D x1,x2,x3;
	int i, step, numsteps = 10;
	float center = SQUARE_SIZE / 2;

	x1.x = x1.z = 0.0;

	for(i = 0;i < 4;i++){
		x2.x = 0.0;
		x2.z = center;

		for(step = 1;step <= numsteps;step++){
			x3.x = center + cos(M_PI + (M_PI / (2 * numsteps))*step) * center;
			x3.z = center + sin(M_PI + (M_PI / (2 * numsteps))*step) * center;

			!s.wireframe ? glBegin(GL_TRIANGLES) : glBegin(GL_LINE_LOOP);
				glNormal3f(0.0, 1.0, 0.0);
				glVertex3f(x1.x, 0.0, x1.z);
				glVertex3f(x3.x, 0.0, x3.z);
				glVertex3f(x2.x, 0.0, x2.z);
			glEnd();

			!s.wireframe ? glBegin(GL_QUADS) : glBegin(GL_LINE_LOOP);
				glNormal3f(cos(M_PI + (M_PI / (2 * numsteps))*(step-1)), 0.0, sin(M_PI + (M_PI / (2 * numsteps))*(step-1)));
				glVertex3f(x2.x, 0.0, x2.z);
				glNormal3f(cos(M_PI + (M_PI / (2 * numsteps))*step), 0.0, sin(M_PI + (M_PI / (2 * numsteps))*step));
				glVertex3f(x3.x, 0.0, x3.z);
				glVertex3f(x3.x, -SQUARE_SIZE, x3.z);
				glNormal3f(cos(M_PI + (M_PI / (2 * numsteps))*(step-1)), 0.0, sin(M_PI + (M_PI / (2 * numsteps))*(step-1)));
				glVertex3f(x2.x, -SQUARE_SIZE, x2.z);
			glEnd();

			x2.x = x3.x;
			x2.z = x3.z;
		}

		glTranslatef(SQUARE_SIZE, 0.0, 0.0);
		glRotatef(-90, 0.0, 1.0, 0.0);
	}
}
