#ifndef LABYRINTH_H
#define LABYRINTH_H

/* define constants for some keys */
#define ESCAPE 27
#define ARROW_LEFT 100
#define ARROW_UP 101
#define ARROW_RIGHT 102
#define ARROW_DOWN 103
/* gravity divided with a factor */
#define GRAVITY 0.1783 /* 9.81 / 50 */
/* how fast to rotate the board */
#define ROT_SPEED 60
/* max angle of the board */
#define MAX_ANGLE 5
/* width of the board */
#define BOARD_WIDTH 20
/* Size of each square (the board has BOARD_WIDTH number of squares */
#define SQUARE_SIZE 0.075
/* the space between the board and the box walls */
#define SPACE 0.020

/**
 * A simple struct that can hold a 2d coordinate
 */
typedef struct{
	float x;
	float z;
} coord2D;

/**
 * Ball structure
 */
typedef struct{
	float x; /* x position of the ball */
	float y; /* y position of the ball */
	float z; /* z position of the ball */
	float rad; /* radius of the ball */
	float dia; /* diameter of the ball */
	float circumf; /* circumference of the ball */
	float rotX; /* rotation around the x axis */
	float rotZ; /* rotation around the z axis */
	float curSpeedX; /* the speed of the ball in the x direction */
	float curSpeedY; /* the speed of the ball in the y direction */
	float curSpeedZ; /* the speed of the ball in the z direction */
} ball;

/**
 * Board structure
 */
typedef struct{
	float rotX; /* rotation around the x axis of the board */
	float rotZ; /* rotation around the z axis of the board */
} board;

/**
 * A structure for the walls of the box
 */
typedef struct{
	float width; /* width of one wall */
	float height; /* height of one wall */
	float depth; /* depth of one wall */
} boxwall;

/**
 * Some states used in the game
 */
typedef struct{
	int left_p; /* flag to tell if the left arrow key is pressed */
	int up_p; /* flag to tell if the up arrow key is pressed */
	int right_p; /* flag to tell if the right arrow key is pressed */
	int down_p; /* flag to tell if the down arrow key is pressed */
	int zoom_in; /* flag to tell if we are zooming in */
	int zoom_out; /* flag to tell if we are zooming out */
	int verbose; /* flag to tell if fps is to be printed */
	int wireframe; /* flag to tell if we are to draw wireframe models */
} state;

/**
 * Some info used in the game
 */
typedef struct{
	int window; /* glut window */
	double lastframe; /* used for time measuring */
	double currentframe; /* used for time measuring */
	float zoomFactor; /* initial zoom factor */
	float screenRotX; /* rotation around the x axis for the "view" */
	float screenRotY; /* rotation around the y axis for the "view" */
	float screenRotZ; /* rotation around the z axis for the "view" */
	int lastMouseX; /* last x position of the mouse */
	int lastMouseY; /* last y position of the mouse */
} labyrinth;

void zoomIn();
void zoomOut();
void init(int width, int height);
void idle();
void draw();
void resize(GLsizei width, GLsizei height);
int main(int argc, char **argv);

#endif
