#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>
#include "labyrinth.h"
#include "textures.h"

GLuint texture[NUM_TEXTURES]; /* array to hold texture id's */

/**
 * This function is a modified version of the one found in some
 * of the nehe tutorials on http://nehe.gamedev.net/
 */
int imageLoad(char *filename, Image *image)
{
	FILE *file;
	unsigned long size; /* size of the image in bytes */
	unsigned long i; /* standard counter */
	unsigned short int planes; /* number of planes in image (must be 1) */
	unsigned short int bpp; /* number of bits per pixel (must be 24) */
	char temp; /* used to convert bgr to rgb color. */

	/* make sure the file is there. */
	if((file = fopen(filename, "rb")) == NULL){
		printf("File Not Found : %s\n",filename);
		return 0;
	}

	/* seek through the bmp header, up to the width/height: */
	fseek(file, 18, SEEK_CUR);

	/* read the width */
	if((i = fread(&image->width, 4, 1, file)) != 1) {
		printf("Error reading width from %s.\n", filename);
		return 0;
	}

	/* read the height */
    if((i = fread(&image->height, 4, 1, file)) != 1){
		printf("Error reading height from %s.\n", filename);
		return 0;
	}

	/* calculate the size (assuming 24 bits or 3 bytes per pixel). */
	size = image->width * image->height * 3;

	/* read the planes */
	if((fread(&planes, 2, 1, file)) != 1){
		printf("Error reading planes from %s.\n", filename);
		return 0;
	}

	if(planes != 1){
		printf("Planes from %s is not 1: %u\n", filename, planes);
		return 0;
    }

	/* read the bpp */
	if((i = fread(&bpp, 2, 1, file)) != 1){
		printf("Error reading bpp from %s.\n", filename);
		return 0;
	}

	if(bpp != 24){
		printf("Bpp from %s is not 24: %u\n", filename, bpp);
		return 0;
	}

	/* seek past the rest of the bitmap header. */
	fseek(file, 24, SEEK_CUR);

	/* read the data. */
	image->data = (char *) malloc(size);
	if(image->data == NULL){
		printf("Error allocating memory for color-corrected image data");
		return 0;
	}

	if((i = fread(image->data, size, 1, file)) != 1){
		printf("Error reading image data from %s.\n", filename);
		return 0;
	}

	for(i=0;i<size;i+=3){ /* reverse all of the colors. (bgr -> rgb) */
		temp = image->data[i];
		image->data[i] = image->data[i+2];
		image->data[i+2] = temp;
	}

	return 1;
}

/**
 * Function that loads the textures used in the game
 */
void loadTextures()
{
	Image *textureImages[NUM_TEXTURES];
	char *filenames[NUM_TEXTURES];
	int i;

	for(i = 0;i < NUM_TEXTURES;i++){
		filenames[i] = (char*)malloc(100);
		textureImages[i] = (Image *)malloc(sizeof(Image));
	}

	filenames[0] = "textures/wood1.bmp";
	filenames[1] = "textures/wood2.bmp";
	filenames[2] = "textures/wood3.bmp";
	filenames[3] = "textures/wood4.bmp";
	filenames[4] = "textures/board.bmp";
	filenames[5] = "textures/knob.bmp";

	glGenTextures(NUM_TEXTURES, texture);

	for(i = 0;i < NUM_TEXTURES;i++){
		if(imageLoad(filenames[i],textureImages[i])){
			glBindTexture(GL_TEXTURE_2D, texture[i]);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
			glTexImage2D(GL_TEXTURE_2D, 0, 3, textureImages[i]->width, textureImages[i]->height, 0, GL_RGB, GL_UNSIGNED_BYTE, textureImages[i]->data);

			if (textureImages[i]){
				if (textureImages[i]->data){
					free(textureImages[i]->data);
				}
				free(textureImages[i]);
			}
		}
	}
}
